"""
 Copyright (c) 2021 Faisal Thaheem (https://github.com/faisalthaheem/python-stanag-4586-EDA-v1)
 License GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007
"""

from ctypes import *
from .message_base import MessageBase

class Message200(MessageBase):
    MSGLEN  = 68
    MSGNULL = b"\x00" * MSGLEN

    #Constants for set zoom field
    SET_ZOOM_USE_HV_FOV = 0
    SET_ZOOM_STOP_ZOOM = 1
    SET_ZOOM_ZOOM_IN = 2
    SET_ZOOM_ZOOM_OUT = 3

    #Constants for Altitude type field
    ALTITUDE_TYPE_PRESSURE = 0
    ALTITUDE_TYPE_BARO = 1
    ALTITUDE_TYPE_AGL = 2
    ALTITUDE_TYPE_WSG84 = 3

    #Constants for set focus field
    SET_FOCUS_NO_CHANGE = 0
    SET_FOCUS_FOCUS_CLOSER = 1
    SET_FOCUS_FOCUS_FARTHER = 2

    #Constants for focus type field
    FOCUS_TYPE_AUTO = 0
    FOCUS_TYPE_MANUAL = 1

    _pack_ = 1
    _fields_ = [
        ("time_stamp",                          c_double),
        ("vehicle_id",                          c_int32),
        ("cucs_id",                             c_int32),
        ("station_number",                      c_uint32),
        ("set_centreline_azimuth_angle",        c_float),
        ("set_centreline_elevation_angle",      c_float),
        ("set_zoom",                            c_ubyte),
        ("set_horizontal_fov",                  c_float),
        ("set_vertical_fov",                    c_float),
        ("horizontal_slew_rate",                c_float),
        ("vertical_slew_rate",                  c_float),
        ("latitude",                            c_double),
        ("longitude",                           c_double),
        ("altitude",                            c_float),
        ("altitude_type",                       c_ubyte),
        ("set_focus",                           c_ubyte),
        ("focus_type",                          c_ubyte),
      ]
    
    def __new__(cls, byte_buffer=None):
        return cls.from_buffer_copy(byte_buffer)

    def __init__(self, byte_buffer=None):
        pass

    def encode(self):
        return bytes(self)

    def getStationId(self):
       return self.station_number


"""
 Copyright (c) 2021 Faisal Thaheem (https://github.com/faisalthaheem/python-stanag-4586-EDA-v1)
 License GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007
"""

from ctypes import *
from .message_base import MessageBase

class Message300(MessageBase):
    
    MSGLEN  = 31
    MSGNULL = b"\x00" * MSGLEN

    PAYLOAD_TYPE_UNSPECIFIED = 0x00
    PAYLOAD_TYPE_EO = 0x01
    PAYLOAD_TYPE_IR = 0x02
    PAYLOAD_TYPE_EOIR = 0x03
    PAYLOAD_TYPE_SAR = 0x04
    PAYLOAD_TYPE_FIXED_CAMERA = 0x05
    PAYLOAD_TYPE_COMMS_RELAY = 0x06
    PAYLOAD_TYPE_DISPENSABLE_PAYLOAD = 0x07
    PAYLOAD_TYPE_RECORDER = 0x08
    PAYLOAD_TYPE_PAYLOAD_BAY_DOOR = 0x09
    PAYLOAD_TYPE_CBRN = 0x0A
    PAYLOAD_TYPE_SMS = 0x0B
    
    _pack_ = 1
    _fields_ = [
        ("time_stamp",                          c_double),
        ("vehicle_id",                          c_int32),
        ("cucs_id",                             c_int32),
        ("vsm_id",                              c_int32),
        ("payload_stations_available",          c_uint32),
        ("station_number",                      c_uint32),
        ("payload_type",                        c_ubyte),
        ("station_door",                        c_ubyte),
        ("number_of_payload_recording_devices", c_ubyte),
      ]
    
    def __new__(cls, byte_buffer=None):
        return cls.from_buffer_copy(byte_buffer)

    def __init__(self, byte_buffer=None):
        pass

    def encode(self):
        return bytes(self)

    def getStationId(self):
       return self.station_number

    def payload_type_to_str(self, pl_type):
        str_type = "unspecified"

        if pl_type == self.PAYLOAD_TYPE_EO:
            str_type = "EO"
        elif pl_type == self.PAYLOAD_TYPE_IR:
            str_type = "IR"
        elif pl_type == self.PAYLOAD_TYPE_EOIR:
            str_type = "EO/IR"
        elif pl_type == self.PAYLOAD_TYPE_SAR:
            str_type = "SAR"
        elif pl_type == self.PAYLOAD_TYPE_FIXED_CAMERA:
            str_type = "FIXED CAMERA"
        elif pl_type == self.PAYLOAD_TYPE_COMMS_RELAY:
            str_type = "COMMS RELAY"
        elif pl_type == self.PAYLOAD_TYPE_DISPENSABLE_PAYLOAD:
            str_type = "DISPENSABLE PAYLOAD"
        elif pl_type == self.PAYLOAD_TYPE_RECORDER:
            str_type = "RECORDER"
        elif pl_type == self.PAYLOAD_TYPE_PAYLOAD_BAY_DOOR:
            str_type = "PAYLOAD BAY DOOR"
        elif pl_type == self.PAYLOAD_TYPE_CBRN:
            str_type = "CBRN"
        elif pl_type == self.PAYLOAD_TYPE_SMS:
            str_type = "SMS"                    
        
        return str_type


"""
 Copyright (c) 2021 Faisal Thaheem (https://github.com/faisalthaheem/python-stanag-4586-EDA-v1)
 License GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007
"""
from abc import ABC, abstractmethod
import logging
import os
from ctypes import *
from sys import meta_path

class MessageBaseMeta(type(BigEndianStructure), type(ABC)):
    pass

class MessageBase(BigEndianStructure, ABC, metaclass=MessageBaseMeta):

    def __init__(self) -> None:
        super().__init__()

        DEBUG_LEVEL = os.getenv("STANAG4586_EDAV1_DEBUG_LEVEL")
        self._logger = logging.getLogger(type(self).__name__)
        
        if DEBUG_LEVEL is not None:
            self._logger.setLevel(DEBUG_LEVEL)
        else:
            self._logger.setLevel(logging.CRITICAL)


    def set_string_field(self, field_name, field_max_length, field_value):

        if len(field_value) > field_max_length or len(field_value) == 0:
            return False

        setattr(self, field_name, field_value.encode("ascii"))

        return True

    def get_string_field(self, field_name):
        if not hasattr(self, field_name): 
            return

        return bytes(getattr(self, field_name)).decode('ascii')

    def dump(self):
        for field_name, field_type in self._fields_:
            self._logger.debug("{}=[{}]".format(field_name, getattr(self, field_name)))

    @abstractmethod
    def getStationId(self):
        """Returns the station or controlled station field value or None if field is absent"""
        


"""
 Copyright (c) 2021 Faisal Thaheem (https://github.com/faisalthaheem/python-stanag-4586-EDA-v1)
 License GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007
"""

from ctypes import *
from .message_base import MessageBase

class Message21(MessageBase):
    MSGLEN  = 35
    MSGNULL = b"\x00" * MSGLEN

    CONTROLLED_STATION_MODE_NOT_IN_CONTROL = 0
    CONTROLLED_STATION_MODE_IN_CONTROL = 1

    VEHICLE_TYPE_UAV = 10000
    VEHICLE_TYPE_UGV = 11000
    VEHICLE_TYPE_USV = 12000
    VEHICLE_TYPE_UUV = 13000

    UGV_SUB_TYPE_MULE       = 1000 #LOAD CARRYING
    UGV_SUB_TYPE_SURV       = 1010 #SURVEILLANCE
    UGV_SUB_TYPE_SENTRY     = 1020 #WITH A GUN AND EO
    UGV_SUB_TYPE_SAM        = 1030 #SURFACE TO AIR MISSLE CARRYING

    _pack_ = 1
    _fields_ = [
        ("time_stamp",              c_double),
        ("vehicle_id",              c_int32),
        ("cucs_id",                 c_int32),
        ("vsm_id",                  c_int32),
        ("data_link_id",            c_int32),
        ("loi_authorized",          c_ubyte),
        ("loi_granted",             c_ubyte),
        ("controlled_station",      c_uint32),
        ("controlled_station_mode", c_ubyte),
        ("vehicle_type",            c_ushort),
        ("vehicle_sub_type",        c_ushort),
      ]
    
    def __new__(cls, byte_buffer=None):
        return cls.from_buffer_copy(byte_buffer)

    def __init__(self, byte_buffer=None):
        pass

    def encode(self):
        return bytes(self)

    def getStationId(self):
       return self.controlled_station 

    def vehicle_type_to_str(self, vehicle_type):

        str_type = "unknown"

        if vehicle_type == self.VEHICLE_TYPE_UAV:
            str_type = "UAV"
        elif vehicle_type == self.VEHICLE_TYPE_UGV:
            str_type = "UGV"
        elif vehicle_type == self.VEHICLE_TYPE_USV:
            str_type = "USV"
        elif vehicle_type == self.VEHICLE_TYPE_UUV:
            str_type = "UUV"

        return str_type

    def vehicle_sub_type_to_str(self, vehicle_sub_type):

        str_type = "unknown"

        if vehicle_sub_type == self.UGV_SUB_TYPE_MULE:
            str_type = "MULE"
        elif vehicle_sub_type == self.UGV_SUB_TYPE_SURV:
            str_type = "SURVEILLANCE"
        elif vehicle_sub_type == self.UGV_SUB_TYPE_SENTRY:
            str_type = "SENTRY"
        elif vehicle_sub_type == self.UGV_SUB_TYPE_SAM:
            str_type = "SAM"

        return str_type


"""
 Copyright (c) 2021 Faisal Thaheem (https://github.com/faisalthaheem/python-stanag-4586-EDA-v1)
 License GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007
"""

from ctypes import *
from .message_base import MessageBase

class Message20020(MessageBase):
    """Custom Message 20020 for query response"""
    
    MSGLEN  = 421
    MSGNULL = b"\x00" * MSGLEN

    _pack_ = 1
    _fields_ = [
        ("time_stamp",                          c_double),
        ("vehicle_id",                          c_int32),
        ("cucs_id",                             c_int32),
        ("station_number",                      c_uint32),
        ("requested_query_type",                c_byte),
        ("response",                            c_char*400),
      ]
    
    def __new__(cls, byte_buffer=None):
        return cls.from_buffer_copy(byte_buffer)

    def __init__(self, byte_buffer=None):
        pass

    def encode(self):
        return bytes(self)

    def getStationId(self):
       return self.station_number

    def get_response(self):
        return self.get_string_field("response")

    def set_response(self, val):
        return self.set_string_field("response", 400, "{:<400}".format(val))



"""
 Copyright (c) 2021 Faisal Thaheem (https://github.com/faisalthaheem/python-stanag-4586-EDA-v1)
 License GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007
"""

from ctypes import *
from .message_base import MessageBase

class Message01(MessageBase):
    MSGLEN  = 35
    MSGNULL = b"\x00" * MSGLEN

    BROADCAST_ID = 0xFFFFFFFF
    LOI_02 = 0x01 #Monitoring the UA / PL
    LOI_03 = 0x02 #Controlling the PL
    LOI_04 = 0x04 #Controlling the UA without takeoff/landing
    LOI_05 = 0x08 #Controlling the UA with takeoff/landing

    _pack_ = 1
    _fields_ = [
        ("time_stamp",              c_double),
        ("vehicle_id",              c_int32),
        ("cucs_id",                 c_int32),
        ("vsm_id",                  c_int32),
        ("data_link_id",            c_int32),
        ("vehicle_type",            c_ushort),
        ("vehicle_sub_type",        c_ushort),
        ("requested_handover_loi",  c_ubyte),
        ("controlled_station",      c_uint32),
        ("controlled_station_mode", c_ubyte),
        ("wait_for_vehicle_data_link_transition_coordination_message",      c_ubyte),
      ]
    
    def __new__(cls, byte_buffer=None):
        return cls.from_buffer_copy(byte_buffer)

    def __init__(self, byte_buffer=None):
        pass

    def encode(self):
        return bytes(self)

    def getStationId(self):
       return self.controlled_station 

    def make_discovery_message(self, cucs_id):
        self.vehicle_id = self.BROADCAST_ID
        self.cucs_id = cucs_id
        self.vsm_id = self.BROADCAST_ID
        self.data_link_id = 0
        self.vehicle_type = 0
        self.vehicle_sub_type = 0
        self.requested_handover_loi = 0
        self.controlled_station = self.BROADCAST_ID
        self.controlled_station_mode = 0
        self.wait_for_vehicle_data_link_transition_coordination_message = 0


"""
 Copyright (c) 2021 Faisal Thaheem (https://github.com/faisalthaheem/python-stanag-4586-EDA-v1)
 License GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007
"""

from ctypes import *
from .message_base import MessageBase

class Message1200(MessageBase):
    MSGLEN  = 35
    MSGNULL = b"\x00" * MSGLEN

    _pack_ = 1
    _fields_ = [
        ("time_stamp",              c_double),
        ("vehicle_id",              c_int32),
        ("cucs_id",                 c_int32),
        ("vsm_id",                  c_int32),
        ("data_link_id",            c_int32),
        ("request_type",            c_ubyte),
        ("requested_message",       c_uint32),
        ("requested_field",         c_ubyte),
        ("station_number",          c_uint32),
        ("sensor_select",           c_ubyte),
      ]
    
    def __new__(cls, byte_buffer=None):
        return cls.from_buffer_copy(byte_buffer)

    def __init__(self, byte_buffer=None):
        pass

    def encode(self):
        return bytes(self)

    def getStationId(self):
       return self.station_number 